@@ -1,3 +1,4 @@
+from __future__ import annotations
 from typing import Optional
 
 import numpy as np
@@ -25,6 +26,8 @@ def cut_adj(
     The cut represents the limiting value of the affinity matrix for
     elements to form edges in the adjacency matrix.
 
+    :group: matrix
+
     Parameters
     ----------
     matrix : array
@@ -87,6 +90,8 @@ def knn_adj(
     towards the k nearest neighbours, determined from the input
     affinity matrix.
 
+    :group: matrix
+
     Parameters
     ----------
     matrix : 2d numpy array
@@ -144,7 +149,20 @@ def knn_adj(
 def fc_adj(
     num_nodes: int, self_loop: bool = False, dtype: npt.DTypeLike = _types.bool
 ) -> Vector:
-    """Create a fully connected adjacency matrix."""
+    """Create a fully connected adjacency matrix.
+
+    :group: matrix
+
+    Parameters
+    ----------
+    num_nodes : int
+        Number of nodes the graph should have.
+    self_loop : bool
+        Whether to include edges from a node to itself. Default is
+        ``False``.
+    dtype : dtype-like
+        The dtype of the output array. Default is ``np.bool_``.
+    """
     adj = np.ones((num_nodes, num_nodes), dtype=dtype)
     if self_loop is False:
         np.fill_diagonal(adj, 0)
@@ -152,11 +170,25 @@ def fc_adj(
 
 
 def delta_R_aff(pmu: gcl.MomentumArray) -> Vector:
-    """Returns a symmetric matrix of delta R vals from input
-    MomentumArray.
+    """Returns the inter-particle Euclidean distances between particles
+    internally within the given MomentumArray.
+
+    :group: matrix
+
+    Parameters
+    ----------
+    pmu : gcl.MomentumArray
+        Four-momenta.
+
+    Returns
+    -------
+    delta_R_matrix : np.ndarray[double]
+        Square symmetric matrix representing the Euclidean distance
+        between every pair of particles in the eta-phi plane.
+
+    Notes
+    -----
+    Infinite values may be encountered if particles are travelling
+    parallel to the beam axis, __ie.__ with infinite pseudorapidity.
     """
-    eta = pmu.eta
-    deta = eta[:, np.newaxis] - eta
-    phi_pol = gcl.calculate.phi_pol(pmu, normalize=False)
-    dphi = np.angle(phi_pol[:, np.newaxis] * phi_pol.conjugate())
-    return np.hypot(deta, dphi)
+    return pmu.delta_R(pmu)
